"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.priority = exports.type = exports.name = void 0;
exports.attach = attach;
const node_assert_1 = require("node:assert");
const debug_1 = __importDefault(require("debug"));
const fromEvent_1 = __importDefault(require("promise-toolbox/fromEvent"));
const ioredis_1 = require("ioredis");
const path_1 = require("path");
const utils_1 = require("@microfleet/utils");
const common_errors_1 = require("common-errors");
const plugin_redis_core_1 = require("@microfleet/plugin-redis-core");
const debug = (0, debug_1.default)('mservice:redisCluster');
/**
 * Plugin name
 */
exports.name = 'redis';
/**
 * Plugin type
 */
exports.type = utils_1.PluginTypes.database;
/**
 * Relative priority inside the same plugin group type
 */
exports.priority = 10;
/**
 * Attaches Redis Cluster plugin.
 * @param  [conf={}] - Configuration for Redis Cluster Connection.
 * @returns Connections and Destructors.
 */
async function attach(opts = {}) {
    (0, node_assert_1.strict)(this.hasPlugin('validator'), new common_errors_1.NotFoundError('validator module must be included'));
    await this.validator.addLocation((0, path_1.resolve)(__dirname, '../schemas'));
    const isClusterStarted = (0, plugin_redis_core_1.isStarted)(this, ioredis_1.Cluster);
    const conf = this.validator.ifError('redisCluster', opts);
    this.redisType = plugin_redis_core_1.REDIS_TYPE_CLUSTER;
    this.log.debug({ conf }, 'redis config');
    this.redis = new ioredis_1.Cluster(conf.hosts, { ...conf.options, lazyConnect: true });
    return {
        /**
         * @returns Opens redis connection.
         */
        async connect() {
            const $conn = this.redis.connect();
            const $ready = (0, fromEvent_1.default)(this.redis, 'ready', { ignoreErrors: true });
            await Promise.race([$conn, $ready]);
            // attach to instance right away
            if (conf.luaScripts) {
                debug('attaching lua');
                await (0, plugin_redis_core_1.loadLuaScripts)(this, conf.luaScripts, this.redis);
            }
            this.addMigrator('redis', plugin_redis_core_1.performMigration, this.redis, this);
            this.emit('plugin:connect:redisCluster', this.redis);
            return this.redis;
        },
        /**
         * @returns Returns current status of redis cluster.
         */
        status: plugin_redis_core_1.hasConnection.bind(this, isClusterStarted),
        /**
         * @returns Closes redis connection.
         */
        async close() {
            (0, node_assert_1.strict)(isClusterStarted(), plugin_redis_core_1.ERROR_NOT_STARTED);
            try {
                await this.redis.quit();
            }
            catch (e) {
                if (e.message === 'Connection is closed.') {
                    return;
                }
                throw e;
            }
            this.emit('plugin:close:redisCluster');
        },
    };
}
//# sourceMappingURL=redis-cluster.js.map